@@ -1,7 +1,7 @@
 #------------------------------------------------
-# Rnmr1D package: Build 1r spectrum from FID file (Bruker/Varian/nmrML)
-# Project: NMRProcFlow - MetaboHUB
-# (C) 2015 - D. JACOB - IMRA UMR1332 BAP
+# Rnmr1D package: Build 1r spectrum from FID file (Bruker/RS2D/Varian/nmrML)
+# Project: NMRProcFlow
+# (C) 2019 - D. JACOB - IMRA UMR1332 BAP
 #------------------------------------------------
 
 #' Spec1rDoProc
@@ -23,7 +23,7 @@ Spec1rDoProc <- function(Input, param=Spec1rProcpar)
 #' \itemize{
 #'   \item \code{DEBUG} : Debug - defaut value = TRUE
 #'   \item \code{LOGFILE} : Messages output file - default value = ""
-#'   \item \code{VENDOR} : Instrumental origin of the raw data (bruker, varian) - default value = 'bruker'
+#'   \item \code{VENDOR} : Instrumental origin of the raw data (bruker, varian, jeol, rs2d) - default value = 'bruker'
 #'   \item \code{READ_RAW_ONLY} : Read Raw file only; do not carry out processing; if raw file is depending on INPUT_SIGNAL - default value = FALSE
 #'   \item \code{INPUT_SIGNAL} : What type of input signal: 'fid' or '1r' - default value = 'fid'
 #'   \item \code{PDATA_DIR} : subdirectory containing the 1r file (bruker's format only) - default value = 'pdata/1'
@@ -121,6 +121,19 @@ Spec1rProcpar <- list (
    acqval
 }
 
+#### Retrieve a parameter value from RS2D acquisition parameters
+#--  internal routine
+#--  ACQ: list of acquisition parameters of the acqus file
+#--  paramStr: name of the parameter
+.rs2d.get_param <- function (ACQ,paramStr,type="numeric")
+{
+   regexpStr <- paste("^",paramStr,"=",sep="")
+   acqval <- gsub("^[^=]+= ?","" ,ACQ[which(simplify2array(regexpr(regexpStr,ACQ))>0)])
+   if (type=="numeric")
+       acqval <-as.numeric(acqval)
+   acqval
+}
+
 ### Estime Group Delay
 #-- fid : free induction decay - Complex data type
 
@@ -151,11 +164,11 @@ Spec1rProcpar <- list (
    # FID filename
    FIDFILE <- "fid"
    if (!file.exists(FIDFILE)) 
-       stop("Invalide data type : File ", FIDFILE, " does not exist\n")
+       stop("FID File ", FIDFILE, " does not exist\n")
    # Acquisition parameters filename
    ACQFILE <- "acqus"
    if (!file.exists(ACQFILE)) 
-       stop("Invalide data type : Acquisition parameter File (", ACQFILE, ") does not exist\n")
+       stop("Acquisition parameter File (", ACQFILE, ") does not exist\n")
 
    ACQ     <- readLines(ACQFILE)
    PROBE   <- .bruker.get_param(ACQ,"PROBHD",type="string")
@@ -228,7 +241,7 @@ Spec1rProcpar <- list (
 
    acq <- list( INSTRUMENT=INSTRUMENT, SOFTWARE=SOFTWARE, ORIGIN=ORIGIN, ORIGPATH=ORIGPATH, 
                 PROBE=PROBE, PULSE=PULSE, SOLVENT=SOLVENT, TEMP=TEMP, NUC=NUC, 
-                NUMBEROFSCANS=NUMBEROFSCANS, DUMMYSCANS=DUMMYSCANS,
+                NUMBEROFSCANS=NUMBEROFSCANS, DUMMYSCANS=DUMMYSCANS, OFFSET=O1/SFO1,
                 RELAXDELAY=RELAXDELAY, SPINNINGRATE=SPINNINGRATE, PULSEWIDTH=PULSEWIDTH,
                 TD=TD, SW=SW, SWH=SWH, SFO1=SFO1, O1=O1, GRPDLY=GRPDLY )
    spec <- list( path=DIR, acq=acq, fid=fid )
@@ -236,6 +249,74 @@ Spec1rProcpar <- list (
    spec
 }
 
+#### Read FID data and the main parameters needed to generate the real spectrum
+#--  internal routine
+#-- DIR: RS2D directory containing the FID
+.read.FID.rs2d <- function(DIR)
+{
+   cur_dir <- getwd()
+   setwd(DIR)
+
+   # FID filename
+   FIDFILE <- "data.dat"
+   if (!file.exists(FIDFILE)) 
+       stop("DATA File ", FIDFILE, " does not exist\n")
+   # Acquisition parameters filename
+   ACQFILE <- "header.xml"
+   if (!file.exists(ACQFILE)) 
+       stop("Acquisition parameter File (", ACQFILE, ") does not exist\n")
+   ACQ <- NULL
+   tree <- xmlTreeParse(ACQFILE)
+   root <- xmlRoot(tree)
+   xmlParams <- xmlElementsByTagName(root, "entry", recursive = TRUE)
+   for(i in 1:length(xmlParams)) {
+      L <- xmlToList(xmlElementsByTagName(xmlParams[[i]], "value", recursive = FALSE)[[1]])
+      ACQ <- c(ACQ,paste0(L$name,"=",L$value))
+   }
+
+   PROBE   <- .rs2d.get_param(ACQ,"PROBES",type="string")
+   SOLVENT <- .rs2d.get_param(ACQ,"SOLVENT",type="string")
+   PULSE   <- .rs2d.get_param(ACQ,"SEQUENCE_NAME",type="string")
+   TEMP    <- .rs2d.get_param(ACQ,"SAMPLE_TEMPERATURE")
+   RELAXDELAY  <- .rs2d.get_param(ACQ,"Last delay")
+   PULSEWIDTH <- .rs2d.get_param(ACQ,"P1.width")*1e6
+   SPINNINGRATE <- .rs2d.get_param(ACQ,"SPIN_RATE")
+   NUMBEROFSCANS <- .rs2d.get_param(ACQ,"NUMBER_OF_AVERAGES")
+   DUMMYSCANS <- .rs2d.get_param(ACQ,"DUMMY_SCAN")
+   NUC     <- .rs2d.get_param(ACQ,"OBSERVED_NUCLEUS",type="string")
+   TD      <- .rs2d.get_param(ACQ,"Nb Point")
+   SFO1    <- .rs2d.get_param(ACQ,"OBSERVED_FREQUENCY")/1e6
+   SWH     <- .rs2d.get_param(ACQ,"SPECTRAL_WIDTH")
+   SW      <- SWH/SFO1
+   O1      <- .rs2d.get_param(ACQ,"OFFSET_FREQ_1")
+   OFFSET  <- .rs2d.get_param(ACQ,"SR")/SFO1
+   LOCKPPM <- 0
+   LCKSTATE <- ifelse( .rs2d.get_param(ACQ,"LOCK",type="string")=='true', TRUE, FALSE )
+   ENDIAN  <-  "big"
+   INSTRUMENT <- paste("RS2D",.rs2d.get_param(ACQ,"MODEL_NAME",type="string"))
+   SOFTWARE <-  gsub(" \\[.+\\]","", .rs2d.get_param(ACQ,"SOFTWARE_VERSION",type="string"))
+
+   NP <- 2*TD
+   to.read = file(FIDFILE,"rb")
+   signal<-readBin(to.read, what="double", n=NP, size=4L, endian = ENDIAN)
+   close(to.read)
+   setwd(cur_dir)
+   
+   rawR <- signal[seq(from = 1, to = NP, by = 2)]
+   rawI <- signal[seq(from = 2, to = NP, by = 2)]
+   fid <- complex(real=rawR, imaginary=rawI)
+   TD <- length(fid)
+
+   acq <- list( INSTRUMENT=INSTRUMENT, SOFTWARE=SOFTWARE, ORIGIN="RS2D", ORIGPATH=DIR, 
+                PROBE=PROBE, PULSE=PULSE, SOLVENT=SOLVENT, TEMP=TEMP, NUC=NUC, OFFSET=OFFSET,
+                NUMBEROFSCANS=NUMBEROFSCANS, DUMMYSCANS=DUMMYSCANS, LOCKPPM=LOCKPPM, LCKSTATE=LCKSTATE,
+                RELAXDELAY=RELAXDELAY, SPINNINGRATE=SPINNINGRATE, PULSEWIDTH=PULSEWIDTH,
+                TD=TD, SW=SW, SWH=SWH, SFO1=SFO1, O1=O1, GRPDLY=0 )
+   spec <- list( path=DIR, acq=acq, fid=fid )
+
+   spec
+}
+
 #### Read FID data and the main parameters needed to generate the real spectrum
 #--  internal routine
 #-- DIR: Varian directory containing the FID
@@ -247,11 +328,11 @@ Spec1rProcpar <- list (
    # FID filename
    FIDFILE <- "fid"
    if (!file.exists(FIDFILE)) 
-       stop("Invalide data type : File ", FIDFILE, " does not exist\n")
+       stop("FID File ", FIDFILE, " does not exist\n")
    # Acquisition parameters filename
    ACQFILE <- "procpar"
    if (!file.exists(ACQFILE)) 
-       stop("Invalide data type : Acquisition parameter File (", ACQFILE, ") does not exist\n")
+       stop("Acquisition parameter File (", ACQFILE, ") does not exist\n")
 
    ACQ  <- readLines(ACQFILE)
    SWH  <-  .varian.get_param(ACQ,"sw")
@@ -346,7 +427,6 @@ Spec1rProcpar <- list (
 #-- FILE: JEOL JDF file containing the FID
 .read.FID.jeol <- function(FILE)
 {
-
    if (!file.exists(FILE))
        stop("File ", FILE, " does not exist\n")
 
@@ -661,7 +741,7 @@ Spec1rProcpar <- list (
 
    acq <- list( INSTRUMENT=INSTRUMENT, SOFTWARE=SOFTWARE, ORIGIN=ORIGIN, ORIGPATH=ORIGPATH, PROBE=PROBE, PULSE=PULSE, SOLVENT=SOLVENT,
                 RELAXDELAY=RELAXDELAY, SPINNINGRATE=SPINNINGRATE, PULSEWIDTH=PULSEWIDTH, TEMP=TEMP, NUC=NUC,
-                NUMBEROFSCANS='-', DUMMYSCANS='-', TD=TD, SW=SW, SWH=SWH, SFO1=SFO1, O1=O1, GRPDLY=GRPDLY )
+                NUMBEROFSCANS='-', DUMMYSCANS='-', TD=TD, SW=SW, SWH=SWH, SFO1=SFO1, O1=O1, OFFSET=O1/SFO1, GRPDLY=GRPDLY )
    spec <- list( path=filename, acq=acq, fid=fid )
 
    spec
@@ -670,25 +750,25 @@ Spec1rProcpar <- list (
 #### Read 1r data and the main parameters needed to generate the real spectrum
 #--  internal routine
 #-- DIR: bruker directory containing the 1r
-.read.1r.bruker <- function(DIR, param=Spec1rProcpar) {
-
+.read.1r.bruker <- function(DIR, param=Spec1rProcpar)
+{
    cur_dir <- getwd()
    setwd(DIR)
    
    # Acquisition parameters filename
    ACQFILE <- "acqus"
    if (!file.exists(ACQFILE)) 
-       stop("Invalide data type : Acquisition parameter File (", ACQFILE, ") does not exist\n")
+       stop("Acquisition parameter File (", ACQFILE, ") does not exist\n")
 
    # 1r filename
    SPECFILE <- paste(param$PDATA_DIR,"/1r",sep="")
    if (!file.exists(SPECFILE))
-       stop("Invalide data type : File (", SPECFILE, ") does not exist\n")
+       stop("1r File (", SPECFILE, ") does not exist\n")
 
    # Processing parameters filename
    PROCFILE <- paste(param$PDATA_DIR,"/procs",sep="")
    if (!file.exists(PROCFILE)) 
-       stop("Invalide data type : Acquisition parameter File (", PROCFILE, ") does not exist\n")
+       stop("Processing parameter File (", PROCFILE, ") does not exist\n")
 
    # Read Acquisition parameters
    ACQ     <- readLines(ACQFILE)
@@ -757,6 +837,93 @@ Spec1rProcpar <- list (
    spec
 }
 
+#### Read 1r data and the main parameters
+#--  internal routine
+#-- DIR: rs2d directory containing the experience
+.read.1r.rs2d <- function(DIR, param=Spec1rProcpar)
+{
+   cur_dir <- getwd()
+   RAWDIR <- file.path(DIR,param$PDATA_DIR)
+   if (!file.exists(RAWDIR))
+       stop("Invalide folder : ", RAWDIR, " does not exist\n")
+
+   setwd(RAWDIR)
+   
+   # FID filename
+   FIDFILE <- "data.dat"
+   if (!file.exists(FIDFILE)) 
+       stop("DATA File ", FIDFILE, " does not exist\n")
+   # Acquisition parameters filename
+   ACQFILE <- "header.xml"
+   if (!file.exists(ACQFILE)) 
+       stop("Acquisition parameter File (", ACQFILE, ") does not exist\n")
+   ACQ <- NULL
+   tree <- xmlTreeParse(ACQFILE)
+   root <- xmlRoot(tree)
+   xmlParams <- xmlElementsByTagName(root, "entry", recursive = TRUE)
+   for(i in 1:length(xmlParams)) {
+      L <- xmlToList(xmlElementsByTagName(xmlParams[[i]], "value", recursive = FALSE)[[1]])
+      ACQ <- c(ACQ,paste0(L$name,"=",L$value))
+   }
+
+   PROBE   <- .rs2d.get_param(ACQ,"PROBES",type="string")
+   SOLVENT <- .rs2d.get_param(ACQ,"SOLVENT",type="string")
+   PULSE   <- .rs2d.get_param(ACQ,"SEQUENCE_NAME",type="string")
+   TEMP    <- .rs2d.get_param(ACQ,"SAMPLE_TEMPERATURE")
+   RELAXDELAY  <- .rs2d.get_param(ACQ,"Last delay")
+   PULSEWIDTH <- .rs2d.get_param(ACQ,"P1.width")*1e6
+   SPINNINGRATE <- .rs2d.get_param(ACQ,"SPIN_RATE")
+   NUMBEROFSCANS <- .rs2d.get_param(ACQ,"NUMBER_OF_AVERAGES")
+   DUMMYSCANS <- .rs2d.get_param(ACQ,"DUMMY_SCAN")
+   NUC     <- .rs2d.get_param(ACQ,"OBSERVED_NUCLEUS",type="string")
+   TD      <- .rs2d.get_param(ACQ,"Nb Point")
+   SFO1    <- .rs2d.get_param(ACQ,"OBSERVED_FREQUENCY")/1e6
+   SWH     <- .rs2d.get_param(ACQ,"SPECTRAL_WIDTH")
+   SW      <- SWH/SFO1
+   O1      <- .rs2d.get_param(ACQ,"OFFSET_FREQ_1")
+   OFFSET  <- .rs2d.get_param(ACQ,"SR")/SFO1
+   LOCKPPM <- 0
+   LCKSTATE <- ifelse( .rs2d.get_param(ACQ,"LOCK",type="string")=='true', TRUE, FALSE )
+   ENDIAN  <-  "big"
+   INSTRUMENT <- paste("RS2D",.rs2d.get_param(ACQ,"MODEL_NAME",type="string"))
+   SOFTWARE <-  gsub(" \\[.+\\]","", .rs2d.get_param(ACQ,"SOFTWARE_VERSION",type="string"))
+   GRPDLY <- 0
+
+   # Read Processing parameters
+   SI <- TD
+   PHC0 <-  .rs2d.get_param(ACQ,"PHASE_0")
+   PHC1 <-  .rs2d.get_param(ACQ,"PHASE_1")
+
+   # Read the 1r spectrum
+   NP <- 2*TD
+   to.read = file(FIDFILE,"rb")
+   signal<-readBin(to.read, what="double", n=NP, size=4L, endian = ENDIAN)
+   close(to.read)
+   setwd(cur_dir)
+
+   rawR <- signal[seq(from = 1, to = NP, by = 2)]
+   rawI <- signal[seq(from = 2, to = NP, by = 2)]
+
+   dppm <- SW/(SI-1)
+   pmin <- OFFSET - SW/2
+   pmax <- OFFSET + SW/2
+   ppm <- seq(from=pmin, to=pmax, by=dppm)
+
+   acq <- list( INSTRUMENT=INSTRUMENT, SOFTWARE=SOFTWARE, ORIGIN="RS2D", ORIGPATH=RAWDIR, 
+                PROBE=PROBE, PULSE=PULSE, NUC=NUC, SOLVENT=SOLVENT, TEMP=TEMP, 
+                RELAXDELAY=RELAXDELAY, SPINNINGRATE=SPINNINGRATE, PULSEWIDTH=PULSEWIDTH,
+                TD=TD, SW=SW, SWH=SWH, SFO1=SFO1, O1=O1, GRPDLY=GRPDLY )
+
+   proc <- list( phc0=PHC0, phc1=PHC1, SI=SI )
+
+   param$ZEROFILLING <- FALSE
+   param$LINEBROADENING <- FALSE
+
+   spec <- list( path=DIR, param=param, acq=acq, proc=proc, fid=NULL, int=rev(rawR), dppm=dppm, pmin=pmin, pmax=pmax, ppm=ppm )
+
+   spec
+}
+
 ### Group Delay correction
 .groupDelay_correction <- function(spec, param=Spec1rProcpar)
 {
@@ -863,35 +1030,27 @@ Spec1rProcpar <- list (
     }
     param$SI <- length(rawspec)
 
-    proc <- list( phc0=0, phc1=0, RMS=0, SI=length(rawspec) )
+    proc <- list( phc0=0, phc1=0, RMS=0, SI=length(rawspec), CFRACPPM=param$CFRACPPM)
 
     # PPM Calibration
     m <- proc$SI
     SW <- spec$acq$SW
     SWH <- spec$acq$SWH
     if (param$O1RATIO==1) {
-        O1 <- spec$acq$O1
+        offset <- spec$acq$OFFSET
     } else {
-        O1 <- SWH*param$O1RATIO
+        offset <- SW*param$O1RATIO
     }
     spec$dppm <- SW/(m-1)
     spec$pmin <- 1 - round(SW/5) # 5 - 0.5*SW
     
     repeat {
-       if (toupper(spec$acq$INSTRUMENT) %in% c("JEOL","VARIAN")) {
-          if (param$O1RATIO==1) {
-              spec$pmin <- spec$acq$OFFSET - SW/2
-          } else {
-              spec$pmin <- SW*param$O1RATIO - SW/2
-          }
-          break
-       }
        if (spec$acq$NUC %in% c('1H','H1') ) {
-          spec$pmin <- (O1/SWH-0.5)*SW
+          spec$pmin <- offset - SW/2
           break
        }
        if (spec$acq$NUC == '13C') {
-          spec$pmin <- (O1/SWH-0.5)*SW
+          spec$pmin <- offset - SW/2
           break
        }
        break
@@ -900,7 +1059,6 @@ Spec1rProcpar <- list (
     spec$ppm <- seq(from=spec$pmin, to=spec$pmax, by=spec$dppm)
 
     ### Save into the spec object instance
-    spec$acq$TD <- length(rawspec)
     spec$data <- rawspec
     spec$param <- param
     spec$proc <- proc
@@ -1044,7 +1202,7 @@ Spec1rProcpar <- list (
    # Calibration based on TSP
    if (spec$param$TSP) {
       x0 <- abs(spec$pmin)/SW
-      n1 <- round(m*(x0-0.3/SW)); n2 <- round(m*(x0+0.3/SW))
+      n1 <- round(m*(x0-0.4/SW)); n2 <- round(m*(x0+0.2/SW))
       range <- c(n1:n2)
       if (max(spec$int[ range ])>10*C_estime_sd(spec$int,128)) {
           n0 <- which(spec$int[ range ] == max(spec$int[ range ])) + n1 - 2
@@ -1087,6 +1245,14 @@ Spec1rProcpar <- list (
          spec <- .read.1r.bruker(Input,param)
          break
       }
+      if (param$VENDOR == "rs2d" && param$INPUT_SIGNAL == "fid") {
+         spec <- .read.FID.rs2d(Input)
+         break
+      }
+      if (param$VENDOR == "rs2d" && param$INPUT_SIGNAL == "1r") {
+         spec <- .read.1r.rs2d(Input,param)
+         break
+      }
       if (param$VENDOR == "varian"){
          param$INPUT_SIGNAL <- "fid"
          spec <- .read.FID.varian(Input)
@@ -1126,6 +1292,7 @@ Spec1rProcpar <- list (
 
           # Get real spectrum
           spec$int <- ajustBL(Re(spec$data),0)
+          spec$data <- NULL
 
           # PPM calibration based on TSP
           if (param$TSP) {
@@ -1163,13 +1330,13 @@ Spec1rProcpar <- list (
 #   x,y : spectra data (ppm, intensity)
 #   xlim, ylim : limits (ranges) of ppm(xlim) and intensity (ylim)
 #   tille : origin path of the spectra
-.Finalize <- function(spec, ppm = c(spec$ppm[1], spec$ppm[spec$acq$TD]), ratio=1, reverse=TRUE)
+Finalize <- function(spec, ppm = c(spec$ppm[1], spec$ppm[spec$proc$SI]), ratio=1, reverse=TRUE)
 {
-   if (spec$proc$TSP) spec <- .ppm_calibration(spec)
+   if (spec$param$TSP) spec <- .ppm_calibration(spec)
 
    # Get real spectrum
    spec.int <- spec$int
-   if (spec$proc$RABOT) {
+   if (spec$param$RABOT) {
        V <- stats::quantile( spec.int[ spec.int < 0 ], 0.25 )
        spec.int[ spec.int < V ] <- V
        spec.int <- ajustBL(spec.int,0)
@@ -1203,6 +1370,7 @@ printSpectrum = function(obj)
         "SOLVENT   = ", obj$acq$SOLVENT, "\n",
         "GRPDLY    = ", obj$acq$GRPDLY, "\n",
         "TD        = ", obj$acq$TD, "\n",
+        "SI        = ", obj$proc$SI, "\n",
         "SW        = ", obj$acq$SW, "\n",
         "SWH       = ", obj$acq$SWH, "\n",
         "SFO1      = ", obj$acq$SFO1, "\n",
@@ -1219,10 +1387,10 @@ printSpectrum = function(obj)
 #   ratio  : ratio between the max intensity of the plot, and the max intensity of the spectrum; default value = 1
 #   ppm    : ppm window of the plot; default = all the spectrum
 #   active : 0 => Open a new graphic window; 1 => put in the active graphic window (replace the content)
-plotSpectrum = function(obj, ppm = c(obj$ppm[1], obj$ppm[obj$acq$TD]), ratio=1, title="", 
+plotSpectrum = function(obj, ppm = c(obj$ppm[1], obj$ppm[obj$proc$SI]), ratio=1, title="", 
                          reverse=TRUE, active=FALSE, col="blue", overlay=FALSE, ovlCol="green" )
 {
-   g <- .Finalize(obj, ppm, ratio=ratio)
+   g <- Finalize(obj, ppm, ratio=ratio)
    if (nchar(title)==0) title <- g$title
    if (overlay) {
       graphics::lines( g$x, g$y, col=ovlCol )
@@ -1241,11 +1409,11 @@ writeSpec = function(spec, outdir, mode="bin", name="1r")
    if (nchar(outdir)==0) stop("Error: outdir missing. You need to specify the output directory\n")
    binfile <- paste(outdir,name,sep='/')
 
-   if (spec$proc$TSP) spec <- .ppm_calibration(spec)
+   if (spec$param$TSP) spec <- .ppm_calibration(spec)
 
    # Get real spectrum
    spec.int <- spec$int
-   if (spec$proc$RABOT) {
+   if (spec$param$RABOT) {
        V <- stats::quantile( spec.int[ spec.int < 0 ], 0.25 )
        spec.int[ spec.int < V ] <- V
        spec.int <- ajustBL(spec.int,0)
@@ -1269,10 +1437,10 @@ writeSpec = function(spec, outdir, mode="bin", name="1r")
        paste('##@PHC1= ', spec$phc1*2*pi, sep=''),
        '##@PH_mod= 1',
        paste('##@SF= ', spec$acq$SFO1, sep=''),
-       paste('##@SI= ', spec$acq$TD, sep=''),
+       paste('##@SI= ', spec$proc$SI, sep=''),
        '##@SSB= 0',
        paste('##@SW_p= ', spec$acq$SWH, sep=''),
-       paste('##@TDeff= ', spec$acq$TD, sep=''),
+       paste('##@TDeff= ', spec$proc$SI, sep=''),
        '##@WDW= 1',
        paste('##@YMAX_p= ', max(spec$int), sep=''),
        paste('##@YMIN_p= ', min(spec$int), sep=''),
